#include <ctype.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>           /* For atoi(). */
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>           /* To handle signals. */
#include <curses.h>           /* For cool text window support. */
#include <panel.h>

#define LINES_OF_LOCAL 3                   // Size of local window
#define DIVIDER_Y (LINES - LINES_OF_LOCAL) /* Y-value of horizontal line
                                              separating local and remote. */
#define DEFAULT_PORT 4096                  // Name says it all
#define PENDING_CONNECTION_QUEUE_LENGTH 5  // Queue size for pending connections
#define BUFFER_LENGTH 256                  // Send buffer limit
#define ROOM_LIMIT 1024                    // Ultimate limit on room size
#define NAME_LIMIT 32                      // Size limit on nicknames
#define NAME_B_LIMIT (NAME_LIMIT + 2)      /* Size of nickname buffers which
                                              allows for ": " in the
                                              nickname record          */
#define FULL_SERVER 1                      // Server is full -- DisconnectShutdown
#define NICK_IN_USE 2                      // Nickname collision -- Dis.Shutdown
#define SERVER_TERMINATE 3                 // Server quit -- DisconnectShutdown
#define PASS_THROUGH 0                     // Command type: pass string through
#define NICK_BAN     1                     // Command type: ban by nick
#define ADDR_BAN     2                     // Command type: ban by address
#define SERVER_CONNECT   3                 // Command type: connect to other server
#define SERVER_DISCONNECT 4                // Command type: disconnect other server
#define STATE_SIZE   256                   // RC4: Size of state array

/* These need to be global because we don't      *\
\* know how to pass anything to signal handlers. */
int socketFileDescriptor;
int newSocketFileDescriptor;
int maxFD;
WINDOW *localInput, *remoteInput,*disp,*rdisp;
PANEL *li,*ri,*di,*rdi;
int local_first=0,remote_first=0;
struct {
  int type;
  char param[NAME_B_LIMIT];
  struct hostent *hostPointer;
  int port;
} response;


void start_curses() {
  int x_loop;
  initscr();
  cbreak();
  nonl();
  keypad(stdscr, TRUE);
  start_color();
  init_pair(1,COLOR_WHITE,COLOR_BLUE);
  init_pair(2,COLOR_WHITE,COLOR_BLUE);
  init_pair(3,COLOR_BLACK,COLOR_CYAN);
  init_pair(4,COLOR_BLACK,COLOR_CYAN);
  
/*  mvprintw(0, 2, "View Window");*/
  rdisp=subwin(stdscr,1,COLS,0,0);
  wbkgd(rdisp,COLOR_PAIR(4));
  rdi=new_panel(disp);
  update_panels();
  doupdate();  

  mvprintw(DIVIDER_Y, 2, "Send Window");
  disp=subwin(stdscr,1,COLS,DIVIDER_Y,0);
  wbkgd(disp,COLOR_PAIR(3));
  di=new_panel(disp);
  update_panels();
  doupdate();  
  
  move(DIVIDER_Y + 1, 0);
  refresh();
  localInput = subwin(stdscr, LINES_OF_LOCAL - 1,
                      COLS, DIVIDER_Y +1 , 0);
  scrollok(localInput, 1);
  wbkgd(localInput,COLOR_PAIR(1));
  li=new_panel(localInput);
  update_panels();
  doupdate();
  remoteInput = subwin(stdscr, DIVIDER_Y - 2, COLS,
                       1, 0);
  box(remoteInput,0,0);
  scrollok(remoteInput, 1);
  wbkgd(remoteInput,COLOR_PAIR(2));
  ri=new_panel(remoteInput);
  update_panels();
  doupdate();
  touchwin(stdscr);
  refresh();
  werase(localInput);
  wmove(localInput, 0, 0);
  wrefresh(localInput);
  werase(remoteInput);
  wrefresh(remoteInput);
}

void end_curses() {
  delwin(remoteInput);
  delwin(localInput);
  touchwin(stdscr);
  refresh();
  endwin();
}

void quit_safe(int return_code) {
  int fd;
  for (fd = 3; fd <= maxFD; fd++)
    close(fd);
  maxFD = 3;
  exit(return_code);
}

void RemoteWrite(char *string)
{
  if (remote_first == 0) 
  {
    mvwprintw(remoteInput,0, 0, string);
    remote_first = 1;
  }
  else
    wprintw(remoteInput, string);
  wrefresh(remoteInput);
}

void Die(int signal) { /* Got a kill signal. */
  int fileDescriptor;
  end_curses();
  switch (signal) {
  case SIGTERM: printf("\nReceived kill signal. Shutting down.\n");
    break;
  case SIGSEGV: printf("\nHow do you like out Project.\n");
    break;
  case SIGINT: printf("\nReceived keyboard interrupt. Shutting down.\n");
    break;
  default: break;
  };
  for (fileDescriptor = 3; fileDescriptor <= maxFD; fileDescriptor++) {
    close(fileDescriptor);
  };
  exit(0);
}

void DisconnectShutdown(int code) {
  /* The remote host is no longer connected. */
  int fileDescriptor;
  if (code == NICK_IN_USE) sleep(3);
  end_curses();
  for(fileDescriptor = 3; fileDescriptor <= maxFD; fileDescriptor++) {
    close(fileDescriptor);
  };
  switch (code) {
  case FULL_SERVER: printf("\nDisconnected.Server is full,try again later.\n");
    break;
  case NICK_IN_USE: printf("\nThese names are in use. ");
    printf("Try again with a different one.\n");
    break;
  case SERVER_TERMINATE: printf("\nThe server has terminated the session.\n");
    break;
  default: printf("\nRemote host disconnected. Shutting down.\n");
    break;
  };
  exit(0);
}

int RunAsClient(char *hostname, int port, char nickname[]) 
{
  char *str=nickname; 	

  int i;
   
struct hostent *hostPointer;
  struct sockaddr_in socketAddress;
  char buffer[BUFFER_LENGTH], serverNick[NAME_B_LIMIT], Nick[NAME_B_LIMIT];
  int length, numActiveFDs, fileDescriptor;
  fd_set readSet, testSet;

  /* Translate hostname into peer's IP address. */
  hostPointer = gethostbyname(hostname);

  if (!hostPointer) {
    end_curses();
    fprintf(stderr, "chat: unknown host: %s\n", hostname);
    quit_safe(1);
  }
 
  /* Build address data structure. */
  bzero((char *)&socketAddress, sizeof(socketAddress));
  socketAddress.sin_family = AF_INET;
  bcopy(hostPointer->h_addr,
        (char *)&socketAddress.sin_addr,
        hostPointer->h_length);
  socketAddress.sin_port = htons(port);

  /* Active open. */
  if ((socketFileDescriptor = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    end_curses();
    perror("chat: socket");
    quit_safe(1);
  };

  /* Keep track of the largest file descriptor. */
  maxFD = socketFileDescriptor;

  if (connect(socketFileDescriptor,
              (struct sockaddr *)&socketAddress,
              sizeof(socketAddress)) < 0) {
    end_curses();
    perror("chat: connect");
    close(socketFileDescriptor);
    quit_safe(1);
  };

  FD_ZERO(&readSet);
  FD_SET(socketFileDescriptor, &readSet);
  FD_SET(0, &readSet);
RemoteWrite("Establishing connection,kindly wait.");
/*RemoteWrite("|");*/
for(i=1;i<4;i++)
{
	RemoteWrite(".");
	sleep(1);
}
/*RemoteWrite("|");*/

  RemoteWrite("\n***Connection established.***\n");

  /* Send the nickname. */
  strcat(nickname, ": ");
  send(socketFileDescriptor, nickname, strlen(nickname), 0);

  /* Check to see if the server is full. */
  recv(socketFileDescriptor, buffer, sizeof(buffer), 0);
  if (!strcasecmp(buffer, "Server full.")) {
    DisconnectShutdown(FULL_SERVER);
  } else if (!strcasecmp(buffer, "Nickname collision.")) {
    RemoteWrite("\n\n***The following nicks have been taken:\n");
    strcpy(Nick, "\0");
    recv(socketFileDescriptor, serverNick, sizeof(serverNick), 0);
    while (strcasecmp(Nick, serverNick)) {
      recv(socketFileDescriptor, Nick, sizeof(Nick), 0);
      length = (int) strlen(Nick);
      Nick[length] = '\n';
      Nick[length + 1] = '\0';
      RemoteWrite(Nick);
    };
    DisconnectShutdown(NICK_IN_USE);
  };
  
  while (1) {
    testSet = readSet;
    numActiveFDs = select((maxFD + 1), &testSet, (fd_set *)0,                        (fd_set *)0, (struct timeval *)0);
    
    if (numActiveFDs == -1) {
      end_curses();
      perror("chat: select");
      quit_safe(1);
    }
    
    for (fileDescriptor = 0;
         fileDescriptor <= maxFD;
         fileDescriptor++) {
      if (FD_ISSET(fileDescriptor, &testSet)) {
        if (fileDescriptor == socketFileDescriptor) {
          if (recv(socketFileDescriptor, buffer, sizeof(buffer), 0) == 0)
           DisconnectShutdown(SERVER_TERMINATE);
          RemoteWrite(buffer);
          RemoteWrite("\n");
        } else if (fileDescriptor == 0) {
          wgetstr(localInput,buffer);
          send(socketFileDescriptor, buffer, strlen(buffer), 0);
          werase(localInput);
          wmove(localInput, 0, 0);
          wrefresh(localInput);
        };
      };
    };
  };
}

void CommandInterpret(char *commandStr) {
  char param[NAME_B_LIMIT], command[NAME_LIMIT], port[NAME_LIMIT];
  int currentcom = 1, currentparam = 0, currentport = 0;

  while (commandStr[currentcom] != ' ') {
    command[currentcom - 1] = commandStr[currentcom];
    currentcom++;
  };
  command[currentcom - 1] = '\0';

  if (!(strcasecmp(command, "ban"))) {
    response.type = NICK_BAN;
  } else if (!(strcasecmp(command, "aban"))) {
    response.type = ADDR_BAN;
  } else if (!(strcasecmp(command, "connect"))) {
    response.type = SERVER_CONNECT;
  } else if (!(strcasecmp(command, "disconnect"))) {
    response.type = SERVER_DISCONNECT;
  } else {
    response.type = PASS_THROUGH;
    return;
  };

  currentcom++;  
  while (commandStr[currentcom] != '#')
  {
    param[currentparam] = commandStr[currentcom];
    currentcom++;
    currentparam++;
  };


  if (response.type < SERVER_CONNECT && response.type)
  {
    param[currentparam++] = ':';
    param[currentparam++] = ' ';
  }
    response.hostPointer = gethostbyname(param);
	response.port = DEFAULT_PORT;
  param[currentparam] = '\0';
  strcpy(response.param, param);
  return;
}

int RunAsServer(int port, char nickname[], int connectionLimit) {
  struct hostent *hostPointer;
  struct sockaddr_in socketAddress;
  int length, numActiveFDs, fileDescriptor, fd_write, index,nind, strlength, goahead,pfd,i,done=0;
  fd_set readSet, testSet;
  char NickNames[ROOM_LIMIT][NAME_B_LIMIT];
  char sendBuffer[BUFFER_LENGTH + NAME_B_LIMIT], buffer[BUFFER_LENGTH],pbuf[BUFFER_LENGTH],sbuf[BUFFER_LENGTH];
  unsigned int Addr[ROOM_LIMIT], BannedAddr[ROOM_LIMIT], bob, BanTop = 0;

  int  a=system("ifconfig >  /root/serv_adrs");
   a=system("chmod 744 /root/serv_adrs");			
  for (index = 0; index < ROOM_LIMIT; index++) {
    NickNames[index][0] = '\0';
  };

  /* Put the server's nickname in the list. */
  strcat(nickname, ": ");
  strcpy(NickNames[0], nickname);

  /* Set up the address struct. */
  bzero((char *)&socketAddress, sizeof(socketAddress));
  socketAddress.sin_family = AF_INET;
  socketAddress.sin_addr.s_addr = INADDR_ANY;
  socketAddress.sin_port = htons(port);
  /* htons() to deal with byte-ordering crap. */

  /* Create a socket. */
  if ((socketFileDescriptor = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    end_curses();
    perror("chat: socket");
    quit_safe(1);
  }
    
  /* Keep track of the largest file descriptor. */
  maxFD = socketFileDescriptor;

  /* Name the socket. */
  if ((bind(socketFileDescriptor,
            (struct sockaddr *)&socketAddress,
            sizeof(socketAddress))) < 0) {
    end_curses();
    perror("chat: bind");
    quit_safe(1);
  };

  /* Set up the connection queue. */
  listen(socketFileDescriptor, PENDING_CONNECTION_QUEUE_LENGTH);

  /* Initialize the file descriptor sets. */
  FD_ZERO(&readSet);
  FD_ZERO(&testSet);
  FD_SET(socketFileDescriptor, &readSet);
  FD_SET(0, &readSet);  /* Add stdin to the set. */
  
  /* Initialize the address arrays */
  for (bob = 0; bob < ROOM_LIMIT; bob++)
    BannedAddr[bob] = Addr[bob] = 0;

  /* The loop! */
  while (1) {
    testSet = readSet;  /* Use a copy of the set. */

    numActiveFDs = select((maxFD + 1), &testSet, (fd_set *)0,
                          (fd_set *)0, (struct timeval *)0);

    if (numActiveFDs == -1) {
      end_curses();
      perror("chat: select");
      quit_safe(1);
    };

    for (fileDescriptor = 0;
         fileDescriptor <= maxFD;
         fileDescriptor++) {
      if (FD_ISSET(fileDescriptor, &testSet)) {
        if (fileDescriptor == 0) {
          wgetstr(localInput,buffer);
          buffer[(sizeof(buffer) - 1)] = '\0';
          if (buffer[0]== '#') {
            CommandInterpret(buffer);
          }
          else if(buffer[0]=='$')
          {
            index=1,nind=0;
            while(buffer[index]!=' ')
            {
             pbuf[nind]=buffer[index];
             index++;
             nind++;
            }
             pbuf[nind]=' ';
             pbuf[++nind]='\0';
             for(i=0;i<=maxFD && done==0;i++)
             {
             if(strcasecmp(pbuf,NickNames[i])==0)
             {
              nind=0;index++,done=1;
              sendBuffer[0]='\0';
              strcat(sendBuffer,NickNames[0]);
              nind=strlen(NickNames[0]);
              while(buffer[index]!='$')
              {
               sendBuffer[nind]=buffer[index];
               index++;nind++;
              }
              
              pfd=i+3;
              send(pfd,sendBuffer, sizeof(sendBuffer), 0);
              RemoteWrite("Private message sent ");
              RemoteWrite(pbuf);RemoteWrite("\n");
             }
            }//end for
	}//else buffer[0]=$
           else
            response.type = PASS_THROUGH;
          if (response.type == PASS_THROUGH && done==0) {
            sendBuffer[0] = '\0';
            strcat(sendBuffer, NickNames[0]);
            strcat(sendBuffer, buffer);
            RemoteWrite(sendBuffer);
            RemoteWrite("\n");
            for (fd_write = 3; fd_write <= maxFD; fd_write++)
              if (FD_ISSET(fd_write, &readSet) &&
                  fd_write != socketFileDescriptor) {
                send(fd_write, sendBuffer, sizeof(sendBuffer), 0);
              };
          } else if (response.type < SERVER_CONNECT && done==0) {
            for (fd_write = 0;
                 strcasecmp(response.param, NickNames[fd_write]) &&
		   fd_write <= maxFD;
		 fd_write++);
	    if (fd_write < maxFD) {
	      FD_CLR(fd_write + 3, &readSet);
	      if (fd_write + 3 == maxFD) maxFD--;
	      connectionLimit++;
	      close(fd_write + 3);
	      RemoteWrite("*** Banned connection on socket ***\n");
	      sendBuffer[0] = '\0';
	      if (response.type == ADDR_BAN)
		BannedAddr[BanTop++] = Addr[fd_write + 3];
	      Addr[fd_write + 3] = 0;
	    } else {
	      if(done==0)
            {
              RemoteWrite("***Ban failed, perhaps you mispelled the nickname.\n");
	      sendBuffer[0] = '\0';
             }
	    };
	  } 
            else {
	    if(done==0)
	{RemoteWrite("***Command failed, perhaps you mispelled the parameter.\n");
	    sendBuffer[0] = '\0';
 	}		
	  };
          werase(localInput);
          wmove(localInput, 0, 0);
          wrefresh(localInput);
          done=0;
          for(i=0;i<=sizeof(sendBuffer);i++)sendBuffer[i]='\0';
        } else if (fileDescriptor == socketFileDescriptor) {
          /* Accept a new connection. */
          if ((newSocketFileDescriptor =
               accept(socketFileDescriptor,
                      (struct sockaddr *)&socketAddress,
                      &length))
              < 0) {
            end_curses();
            perror("chat: accept");
            quit_safe(1);
          };
	  goahead = 1;
	  for (index = 0; index < sizeof(buffer); index++)
	    buffer[index] = '\0';
	  recv(newSocketFileDescriptor, buffer, sizeof(buffer), 0);
	  if (connectionLimit == 0) {
	    send(newSocketFileDescriptor, "Server full.",
		 sizeof("Server full."), 0);
	    close(newSocketFileDescriptor);
	    goahead = 0;
	  } else if (goahead) {
	    if (strcasecmp(buffer, "\n\0")) {
	      for (index = 0; index < maxFD; index++)
		if (strcasecmp(buffer, NickNames[index]) == 0) {
		  send(newSocketFileDescriptor, "Nickname collision.",
		       sizeof("Nickname collision."), 0);
		  send(newSocketFileDescriptor, NickNames[0],
		       sizeof(NickNames[0]), 0);
		  for (index = maxFD; index > -1; index--)
		    if (FD_ISSET(index, &readSet))
		      send(newSocketFileDescriptor, NickNames[index - 3],
			   sizeof(NickNames[index - 3]), 0);
		  send(newSocketFileDescriptor, NickNames[0],
		       sizeof(NickNames[0]), 0);
		  close(newSocketFileDescriptor);
		  goahead = 0;
		  index = ROOM_LIMIT;
		};
	    };
	  };
	  if (goahead) {
	    for (index = 0; index < ROOM_LIMIT; index++)
	      if (BannedAddr[index] == socketAddress.sin_addr.s_addr) {
		send(newSocketFileDescriptor, "Address banned.",
		     sizeof("Address banned."), 0);
		close(newSocketFileDescriptor);
		goahead = 0;
	      };
	  };
	  if (goahead) {
	    FD_SET(newSocketFileDescriptor, &readSet);
	    /* Keep track of the largest file descriptor. */
	    if (newSocketFileDescriptor > maxFD)
	      maxFD = newSocketFileDescriptor;
	    RemoteWrite("*** New connection on socket from ");

            strlength = (int) strlen(buffer);  /* Add the nickname to the table. */
	    buffer[strlength] = '\0';
	    strcpy(NickNames[newSocketFileDescriptor - 3], buffer);
	    connectionLimit--;             /* Keep track of number of connections */
	    /* Address added to list */
	    Addr[newSocketFileDescriptor - 3] = socketAddress.sin_addr.s_addr;
	    send(newSocketFileDescriptor, "Ok.", sizeof("Ok."), 0);
            buffer[strlength-2]='\0';
            RemoteWrite(buffer);
            RemoteWrite("\n");
	  };
        } else {
          for (index = 0; index < sizeof(buffer); index++)
            buffer[index] = '\0';
          if (recv(fileDescriptor, buffer, sizeof(buffer), 0) == 0) {
            FD_CLR(fileDescriptor, &readSet);
            if (fileDescriptor == maxFD) maxFD--;
            connectionLimit++;
            close(fileDescriptor);
            RemoteWrite("***Closed connection on socket.\n");
	    sendBuffer[0] = '\0';
	    strcpy(sendBuffer, "*** ");
            strcat(sendBuffer, NickNames[fileDescriptor - 3]);
            strcat(sendBuffer, "Goodbye!");
	    RemoteWrite(sendBuffer);
	    RemoteWrite("\n");
            for (fd_write = 3; fd_write <= maxFD; fd_write++)
              if (FD_ISSET(fd_write, &readSet) &&
                  fd_write != socketFileDescriptor) {
                send(fd_write, sendBuffer, sizeof(sendBuffer), 0);
              };
            strcpy(NickNames[fileDescriptor - 3], "");
          } else {
            if(buffer[0]=='$')
            {
             index=1,nind=0;
             while(buffer[index]!=' ')
             {
              pbuf[nind]=buffer[index];
              index++;
              nind++;
             }
             pbuf[nind]=' ';
             pbuf[++nind]='\0';
             for(i=0;i<=maxFD && done==0;i++)
             {
              if(strcasecmp(pbuf,NickNames[i])==0)
              {
               nind=0;index++,done=1;
               sendBuffer[0]='\0';
               strcat(sendBuffer,NickNames[fileDescriptor-3]);
               strcpy(sbuf,sendBuffer);//only for printing sake
               nind=strlen(NickNames[fileDescriptor-3]);
               while(buffer[index]!='$')
               {
                sendBuffer[nind]=buffer[index];
                index++;nind++;
               }
               pfd=i+3;
               send(fileDescriptor,sendBuffer,sizeof(sendBuffer),0);
               if(pfd==3)
               {
                RemoteWrite(sendBuffer);RemoteWrite("\n");
               }
               else
               {
               send(pfd,sendBuffer, sizeof(sendBuffer), 0);
               RemoteWrite("Private message to ");
               RemoteWrite(pbuf); RemoteWrite(" from ");RemoteWrite(sbuf);
		RemoteWrite("\n");
               }
              }//end if
             }//end for
	     done=0;
             for(i=0;i<=sizeof(sendBuffer);i++)sendBuffer[i]='\0';
            }//end if(buffer[0]=$)
            else
              {
              sendBuffer[0] = '\0';
              strcpy(sendBuffer, NickNames[fileDescriptor - 3]);
              strcat(sendBuffer, buffer);
              RemoteWrite(sendBuffer);
              RemoteWrite("\n");
              strlength = strlen(sendBuffer);
              for (fd_write = 3; fd_write <= maxFD; fd_write++)
              if (FD_ISSET(fd_write, &readSet) &&
              fd_write != socketFileDescriptor) {
              send(fd_write, sendBuffer, sizeof(sendBuffer), 0);
              };
            };
          };
        };
      };
    };
  };
}

int main(int argc, char *argv[]) {
  int port, connectionLimit, isnum = 1, index2 = 0;
  char *hostname, nickname[NAME_B_LIMIT];
       char f_string[NAME_B_LIMIT + 30];
  /* Curses stuff. */
  start_curses();
  if (argc < 3) {
    end_curses();
    fprintf(stderr,
            "usage: %s (<hostname> | -server) [port] nickname [room-size]\n",
            argv[0]);
    quit_safe(1);
  };

  /* The almighty sig handlers, powerful and useful, especially
     to prevent icky core dumps. */
  signal(SIGINT, Die);  /* If we get a SIGINT, die. */
  signal(SIGTERM, Die); /* If we get a SIGTERM, die. */
  signal(SIGSEGV, Die); /* To handle those pesky segfaults. */

  if (!strcasecmp(argv[1], "-server")) {
    switch (argc) {
    case 3: port = DEFAULT_PORT; 
      strncpy(nickname, argv[2], NAME_LIMIT);
      connectionLimit = -1;
      break;
    case 4:
      while (index2 < strlen(argv[2])) {
        if (isalpha(argv[2][index2])) {
          isnum = 0;
        };
        index2++;
      };
      if (isnum == 1) {
        port = atoi(argv[2]);
        strncpy(nickname, argv[3], NAME_LIMIT);
        connectionLimit = -1;
      } else {
        port = DEFAULT_PORT;
        strncpy(nickname, argv[2], NAME_LIMIT);
        connectionLimit = atoi(argv[3]);
      }; break;
    case 5:
      port = atoi(argv[2]);
      strncpy(nickname, argv[3], NAME_LIMIT);
      connectionLimit = atoi(argv[4]);
      break;
    };
    
       sprintf(f_string,"The Supreme %s's Interface Window",nickname);
       mvprintw(0, 20, f_string);
       wbkgd(rdisp,COLOR_PAIR(4));
       rdi=new_panel(disp);
       update_panels();
       doupdate();  
      
    RunAsServer(port, nickname, connectionLimit);
  }
  else {
    if (argc == 4) {
      port = atoi(argv[2]); /* Set user specified port. */
      strncpy(nickname, argv[3], NAME_LIMIT);
    }
    else {
      port = DEFAULT_PORT;          /* Or set the default port. */
      strcpy(nickname, argv[2]);
    }
      sprintf(f_string,"%s's Interface Window",nickname);
       mvprintw(0, 25, f_string);
       wbkgd(rdisp,COLOR_PAIR(4));
       rdi=new_panel(disp);
       update_panels();
       doupdate();  
      RunAsClient(argv[1], port, nickname);
  }
}
